import Flutter
import UIKit
import AWSPinpoint

public class SwiftAwsPinpointPlugin: NSObject, FlutterPlugin {
    private var pinpoint: AWSPinpoint?
    public static let shared = SwiftAwsPinpointPlugin()
    struct Event {
        static let ACTION_PUSH_NOTIFICATION = "push-notification"
        static let ON_NEW_TOKEN = "onNewToken"
        static let REGISTER_DEVICE_TOKEN = "registerDeviceToken"
        static let DATA = "data"
        static let NOTIFICATION = "notification"
        static let ON_MESSAGE = "onMessage"
        static let ENDPOINT_ID = "endpointId"
        static let INITIALIZE = "initialize"
        static let SET_USER_ID = "setUserId"
        static let START_SESSION = "startSession"
        static let STOP_SESSION = "stopSession"
        static let SUBMIT_EVENTS = "submitEvents"
        static let RECORD_EVENT = "recordEvent"
        static let SUBMIT_LOGIN_EVENT = "submitLoginEvent"
        static let ADD_ATTRIBUTE = "addAttribute"
        static let KEY = "key"
        static let VALUE = "value"
        static let TYPE_LOGIN_EVENT = "_userauth.sign_in"
    }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "aws_pinpoint_plugin", binaryMessenger: registrar.messenger())
    let instance = SwiftAwsPinpointPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method{
    case Event.REGISTER_DEVICE_TOKEN:
        pinpoint?.notificationManager.interceptDidRegisterForRemoteNotifications(withDeviceToken: Data((call.arguments as! String).utf8))
        result("Success")
    case Event.ENDPOINT_ID:
        result(pinpoint?.targetingClient.currentEndpointProfile().endpointId)
  //  case Event.SET_USER_ID:
      //  var currentPoint = pinpoint?.targetingClient.currentEndpointProfile()
        //currentPoint?.user?.userId = call.arguments as! String
        //pinpoint?.targetingClient.update(currentPoint ??)
    default:
        result(FlutterMethodNotImplemented)
    }
  }
    public func setupClient(withLaunchOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) {
      pinpoint = AWSPinpoint(configuration:
        AWSPinpointConfiguration.defaultPinpointConfiguration(launchOptions: launchOptions))
    }
    
    func create(event: String, attributes: [String: String]? = nil, metrics: [String: NSNumber]? = nil) {
      guard let pinpoint = pinpoint else { return }
      
      let event = pinpoint.analyticsClient.createEvent(withEventType: event)
      if let eventAttributes = attributes {
        for (key, attributeValue) in eventAttributes {
          event.addAttribute(attributeValue, forKey: key)
        }
      }
      
      if let eventMetrics = metrics {
        for (key, metricValue) in eventMetrics {
          event.addMetric(metricValue, forKey: key)
        }
      }
      pinpoint.analyticsClient.record(event)
    }
    
    func createMonetizationEvent(productId: String, price: Double, currency: String, quantity: Int) {
      guard let pinpoint = pinpoint else { return }
      let event = pinpoint.analyticsClient.createVirtualMonetizationEvent(withProductId: productId, withItemPrice: price, withQuantity: quantity, withCurrency: currency)
      pinpoint.analyticsClient.record(event)
      uploadEvents()
    }
    
    func uploadEvents() {
      guard let pinpoint = pinpoint else { return }
      pinpoint.analyticsClient.submitEvents()
    }
    
    
}
